searchData={"items":[{"type":"module","title":"NimbleOwnership","doc":"Module that allows you to manage ownership of resources across processes.\n\nThe idea is that you can track ownership of terms (keys) across processes,\nand allow processes to use a key through processes that are already allowed.\n\n```mermaid\nflowchart LR\n  pidA[\"Process A\"]\n  pidB[\"Process B\"]\n  pidC[\"Process C\"]\n  res([\"Resource (with associated metadata)\"])\n\n  pidA -->|Owns| res\n  pidA -->|Allows| pidB\n  pidB -->|Can access| res\n  pidB -->|Allows| pidC\n  pidC -->|Can access| res\n```\n\nA typical use case for such a module is tracking resource ownership across processes\nin order to isolate access to resources in **test suites**. For example, the\n[Mox](https://hexdocs.pm/mox/Mox.html) library uses this module to track ownership\nof mocks across processes (in shared mode).","ref":"NimbleOwnership.html"},{"type":"module","title":"Usage - NimbleOwnership","doc":"To track ownership of resources, you need to start a `NimbleOwnership` server (a process),\nthrough `start_link/1` or `child_spec/1`.\n\nThen, you can allow a process access to a key through `allow/4`. You can then check\nif a PID can access the given key through `fetch_owner/3`.\n\n#","ref":"NimbleOwnership.html#module-usage"},{"type":"module","title":"Metadata - NimbleOwnership","doc":"You can store arbitrary metadata (`t:metadata/0`) alongside each **owned resource**.\nThis metadata is returned together with the owner PID when you call `fetch_owner/3`.","ref":"NimbleOwnership.html#module-metadata"},{"type":"module","title":"Modes - NimbleOwnership","doc":"The ownership server can be in one of two modes:\n\n  * **private** (the default): in this mode, you can only allow access to a key through\n    the owner PID or PIDs that are already allowed to access the key. You can allow PIDs\n    through `allow/4`. This mode is useful when you want to track ownership of resources\n    in concurrent environments (such as in a test suite).\n\n  * **shared**: in this mode, there is only one *shared owner PID* that owns all the keys\n    in the ownership server. Any other PID can read the metadata associated with any key,\n    but it cannot update the metadata (only the shared owner can).\n\n> ##","ref":"NimbleOwnership.html#module-modes"},{"type":"module","title":"Returning to Private Mode {: .warning} - NimbleOwnership","doc":">\n> If the ownership server is in *shared mode* and the shared owner process terminates,\n> the server automatically returns to *private mode*.","ref":"NimbleOwnership.html#module-returning-to-private-mode-warning"},{"type":"function","title":"NimbleOwnership.allow/5","doc":"Allows `pid_to_allow` to use `key` through `owner_pid` (on the given `ownership_server`).\n\nUse this function when `owner_pid` is allowed access to `key`, and you want\nto also allow `pid_to_allow` to use `key`.\n\nThis function return an error in the following cases:\n\n  * When `pid_to_allow` is already allowed to use `key` via **another owner PID**\n    that is not `owner_pid`. In this case, the `:reason` field of the returned\n    `NimbleOwnership.Error` struct is set to `{:already_allowed, other_owner_pid}`.\n\n  * When the ownership server is in [**shared mode**](#module-modes). In this case,\n    the `:reason` field of the returned `NimbleOwnership.Error` struct is set to\n    `:cant_allow_in_shared_mode`.\n\n> ##","ref":"NimbleOwnership.html#allow/5"},{"type":"function","title":"Tracking Callers {: .tip} - NimbleOwnership.allow/5","doc":">\n> The ownership server automatically considers all the direct and indirect \"children\"\n> of an owner PID as allowed to access the owner's keys. By children, we mean processes\n> that have been spawned by the owner PID or by any of its children, which is something\n> we determine by looking at the `:\"$callers\"` key in the process dictionary.\n> This is useful for many standard process kinds. For example, if a process owns a key\n> and starts a task with `Task.start_link/1`, then the task will be allowed to access\n> the key without having to explicitly call `allow/4`.","ref":"NimbleOwnership.html#allow/5-tracking-callers-tip"},{"type":"function","title":"Examples - NimbleOwnership.allow/5","doc":"iex> pid = spawn(fn -> Process.sleep(:infinity) end)\n    iex> {:ok, server} = NimbleOwnership.start_link()\n    iex> NimbleOwnership.get_and_update(server, self(), :my_key, fn _ -> {:updated, _meta = %{}} end)\n    {:ok, :updated}\n    iex> NimbleOwnership.allow(server, self(), pid, :my_key)\n    :ok\n    iex> NimbleOwnership.fetch_owner(server, [pid], :my_key)\n    {:ok, self()}","ref":"NimbleOwnership.html#allow/5-examples"},{"type":"function","title":"NimbleOwnership.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"NimbleOwnership.html#child_spec/1"},{"type":"function","title":"NimbleOwnership.fetch_owner/4","doc":"Gets the owner of `key` through one of the `callers`.\n\nIf one of the `callers` owns `key` or is allowed access to `key`,\nthen this function returns `{:ok, {owner_pid, metadata}}` where `metadata` is the\nmetadata associated with the `key` under the owner.\n\nIf the ownership server is in [**shared mode**](#module-modes), then this function\nreturns `{:shared_owner, shared_owner_pid}` where `shared_owner_pid` is the PID of the\nshared owner. This is regardless of the `callers`.\n\nIf none of the callers owns `key` or is allowed access to `key`, then this function\nreturns `{:error, reason}`.","ref":"NimbleOwnership.html#fetch_owner/4"},{"type":"function","title":"Examples - NimbleOwnership.fetch_owner/4","doc":"iex> pid = spawn(fn -> Process.sleep(:infinity) end)\n    iex> {:ok, server} = NimbleOwnership.start_link()\n    iex> NimbleOwnership.set_mode_to_shared(server, pid)\n    iex> {:shared_owner, owner_pid} = NimbleOwnership.fetch_owner(server, [self()], :whatever_key)\n    iex> pid == owner_pid\n    true\n\n    iex> {:ok, server} = NimbleOwnership.start_link()\n    iex> NimbleOwnership.fetch_owner(server, [self()], :whatever_key)\n    :error","ref":"NimbleOwnership.html#fetch_owner/4-examples"},{"type":"function","title":"NimbleOwnership.get_and_update/5","doc":"Accesses `key` (owned by `owner_pid`) or initializes the ownership.\n\nUse this function for these purposes:\n\n  * to initialize the ownership of a key\n  * to update the metadata associated with a key","ref":"NimbleOwnership.html#get_and_update/5"},{"type":"function","title":"Usage - NimbleOwnership.get_and_update/5","doc":"When `owner_pid` doesn't own `key`, the value passed to `fun` will be `nil`. Otherwise,\nit will be the current metadata associated with `key` under the owner `owner_pid`.\n\n`fun` must return `{get_value, new_meta}`. `owner_pid` will start owning\n`key` and `new_meta` will be the metadata associated with that ownership, or,\nin case `owner_pid` already owned `key`, then the metadata is updated to `new_meta`.\n\nIf this function is successful, the return value is `{:ok, get_value}` where `get_value`\nis the value returned by `fun` in its return tuple. Otherwise, the return value is\n`{:error, reason}` (see also `NimbleOwnership.Error`).\n\n#","ref":"NimbleOwnership.html#get_and_update/5-usage"},{"type":"function","title":"Updating Metadata from an Allowed Process - NimbleOwnership.get_and_update/5","doc":"If you don't directly have access to the owner PID, but you want to update the metadata\nassociated with the owner PID and `key` *from an allowed process*, do this instead:\n\n  1. Fetch the owner of `key` through `fetch_owner/3`.\n  2. Call `get_and_update/4` with the owner PID as `owner_pid`, passing in a callback\n     function that returns the new metadata.\n\n#","ref":"NimbleOwnership.html#get_and_update/5-updating-metadata-from-an-allowed-process"},{"type":"function","title":"Shared Mode - NimbleOwnership.get_and_update/5","doc":"When the ownership server is set to **shared mode**, you can only call this function\nwith `owner_pid` set to the shared owner PID. See [the module documentation](#module-modes).","ref":"NimbleOwnership.html#get_and_update/5-shared-mode"},{"type":"function","title":"NimbleOwnership.get_owned/4","doc":"Gets all the keys owned by `owner_pid` with all their associated metadata.\n\nIf `owner_pid` doesn't own any keys, then this function returns `default`.","ref":"NimbleOwnership.html#get_owned/4"},{"type":"function","title":"Examples - NimbleOwnership.get_owned/4","doc":"iex> owner = spawn(fn -> Process.sleep(:infinity) end)\n    iex> {:ok, server} = NimbleOwnership.start_link()\n    iex> NimbleOwnership.get_and_update(server, owner, :my_key1, fn _ -> {:ok, 1} end)\n    iex> NimbleOwnership.get_and_update(server, owner, :my_key2, fn _ -> {:ok, 2} end)\n    iex> NimbleOwnership.get_owned(server, owner)\n    %{my_key1: 1, my_key2: 2}\n    iex> NimbleOwnership.get_owned(server, self(), :default)\n    :default","ref":"NimbleOwnership.html#get_owned/4-examples"},{"type":"function","title":"NimbleOwnership.set_mode_to_private/1","doc":"Sets the ownership server to *private mode*.\n\nSee [the module documentation](#module-modes) for more information.","ref":"NimbleOwnership.html#set_mode_to_private/1"},{"type":"function","title":"NimbleOwnership.set_mode_to_shared/2","doc":"Sets the ownership server to *shared mode* and sets `shared_owner` as the shared owner.\n\nSee [the module documentation](#module-modes) for more information.","ref":"NimbleOwnership.html#set_mode_to_shared/2"},{"type":"function","title":"NimbleOwnership.start_link/1","doc":"Starts an ownership server.","ref":"NimbleOwnership.html#start_link/1"},{"type":"function","title":"Options - NimbleOwnership.start_link/1","doc":"This function supports all the options supported by `GenServer.start_link/3`, namely:\n\n  * `:name`\n  * `:timeout`\n  * `:debug`\n  * `:spawn_opt`\n  * `:hibernate_after`","ref":"NimbleOwnership.html#start_link/1-options"},{"type":"type","title":"NimbleOwnership.key/0","doc":"Arbitrary key.","ref":"NimbleOwnership.html#t:key/0"},{"type":"type","title":"NimbleOwnership.metadata/0","doc":"Arbitrary metadata associated with an owned `t:key/0`.","ref":"NimbleOwnership.html#t:metadata/0"},{"type":"type","title":"NimbleOwnership.server/0","doc":"Ownership server.","ref":"NimbleOwnership.html#t:server/0"},{"type":"exception","title":"NimbleOwnership.Error","doc":"Exception struct returned by `NimbleOwnership` functions.","ref":"NimbleOwnership.Error.html"},{"type":"type","title":"NimbleOwnership.Error.t/0","doc":"","ref":"NimbleOwnership.Error.html#t:t/0"}],"content_type":"text/markdown"}